#! /usr/bin/env python3
#
# test for automatic varibright->monobright conversion of messages
# try changing varibright to True for smooth brightness transition
#

import asyncio
import monome


class MonobrightGrid(monome.Grid):
    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.varibright = False


class MonobrightApp(monome.GridApp):
    def __init__(self):
        grid = MonobrightGrid()
        super().__init__(grid)

    async def light(self, x, y):
        for i in range(16):
            self.grid.led_level_set(x, y, i)
            await asyncio.sleep(0.1)

    def on_grid_key(self, x, y, s):
        if s == 1:
            asyncio.create_task(self.light(x, y))


async def main():
    loop = asyncio.get_running_loop()
    monobright_app = MonobrightApp()

    def serialosc_device_added(id, type, port):
        print('connecting to {} ({})'.format(id, type))
        asyncio.create_task(monobright_app.grid.connect('127.0.0.1', port))

    serialosc = monome.SerialOsc()
    serialosc.device_added_event.add_handler(serialosc_device_added)

    await serialosc.connect()
    await loop.create_future()

if __name__ == '__main__':
    asyncio.run(main())



#! /usr/bin/env python3

import asyncio
import monome

from life import Life

async def main():
    loop = asyncio.get_running_loop()

    section1 = monome.GridSection((8, 8), (0, 0))
    section2 = monome.GridSection((8, 8), (8, 0))
    splitter = monome.GridSplitter([section1, section2])

    life1 = Life()
    life1.set_grid(section1)

    life2 = Life()
    life2.set_grid(section2)

    def serialosc_device_added(id, type, port):
        print('connecting to {} ({})'.format(id, type))
        asyncio.create_task(splitter.grid.connect('127.0.0.1', port))

    serialosc = monome.SerialOsc()
    serialosc.device_added_event.add_handler(serialosc_device_added)

    await serialosc.connect()
    await loop.create_future()

if __name__ == '__main__':
    asyncio.run(main())



#! /usr/bin/env python3

import asyncio
import monome

from life import Life

class PagesSerialOsc(monome.SerialOsc):
    def __init__(self, app1, app2, loop=None, autoconnect_app=None):
        super().__init__(loop, autoconnect_app)
        self.app1 = app1
        self.app2 = app2

    async def pages_connect(self, port):
        transport, grid = await self.loop.create_datagram_endpoint(monome.Grid, local_addr=('127.0.0.1', 0), remote_addr=('127.0.0.1', port))

        page1 = monome.Page()
        page2 = monome.Page()
        page_manager = monome.SeqPageManager(grid=grid, pages=[page1, page2])

        self.app1.attach(page1)
        self.app2.attach(page2)

        page_manager.connect()

    def on_device_added(self, id, type, port):
        asyncio.create_task(self.pages_connect(port))

async def main():
    loop = asyncio.get_running_loop()

    pages = monome.SumGridPageManager(2)

    life1 = Life()
    life1.set_grid(pages.pages[0])

    life2 = Life()
    life2.set_grid(pages.pages[1])

    def serialosc_device_added(id, type, port):
        print('connecting to {} ({})'.format(id, type))
        asyncio.create_task(pages.grid.connect('127.0.0.1', port))

    serialosc = monome.SerialOsc()
    serialosc.device_added_event.add_handler(serialosc_device_added)

    await serialosc.connect()
    await loop.create_future()

if __name__ == '__main__':
    asyncio.run(main())



#! /usr/bin/env python3

import random
import asyncio
import monome

FADERS_MAX_VALUE = 100

class Faders(monome.GridApp):
    def __init__(self):
        super().__init__() # TODO: prefix

    def on_grid_ready(self):
        self.grid.led_all(0)

        self.row_values = []
        row_value = 0
        for i in range(self.grid.height):
            self.row_values.append(int(round(row_value)))
            row_value += FADERS_MAX_VALUE / (self.grid.height - 1)

        self.values = [random.randint(0, FADERS_MAX_VALUE) for f in range(self.grid.width)]
        self.faders = [asyncio.create_task(self.fade_to(f, 0)) for f in range(self.grid.width)]

    def on_grid_key(self, x, y, s):
        if s == 1:
            self.faders[x].cancel()
            self.faders[x] = asyncio.create_task(self.fade_to(x, self.row_to_value(y)))

    def value_to_row(self, value):
        return sorted([i for i in range(self.grid.height)], key=lambda i: abs(self.row_values[i] - value))[0]

    def row_to_value(self, row):
        return self.row_values[self.grid.height - 1 - row]

    async def fade_to(self, x, new_value):
        while self.values[x] != new_value:
            if self.values[x] < new_value:
                self.values[x] += 1
            else:
                self.values[x] -= 1
            col = [0 if c > self.value_to_row(self.values[x]) else 1 for c in range(self.grid.height)]
            col.reverse()
            self.grid.led_col(x, 0, col)
            await asyncio.sleep(1/100)

async def main():
    loop = asyncio.get_running_loop()
    faders_app = Faders()

    def serialosc_device_added(id, type, port):
        print('connecting to {} ({})'.format(id, type))
        asyncio.create_task(faders_app.grid.connect('127.0.0.1', port))

    serialosc = monome.SerialOsc()
    serialosc.device_added_event.add_handler(serialosc_device_added)

    await serialosc.connect()
    await loop.create_future()

if __name__ == '__main__':
    asyncio.run(main())



#! /usr/bin/env python3

import random
import asyncio
import monome

from perlin_noise import PerlinNoise

class PerinApp(monome.GridApp):
    def __init__(self):
        super().__init__()
        self.noise = PerlinNoise()
        self.buffer = monome.GridBuffer(16, 16)
        self.task = asyncio.create_task(asyncio.sleep(0))

    def on_grid_ready(self):
        self.alive = True
        self.task = asyncio.create_task(self.animate())

    def on_grid_key(self, x, y, s):
        if s == 1:
            self.grid.led_set(x, y, s)

    def on_grid_disconnect(self, *args):
        self.alive = False
        self.task.cancel()

    async def animate(self):
        delta_x = 0
        delta_y = 0
        delta_z = 0

        while self.alive:
            for x in range(16):
                for y in range(16):
                    value = self.noise([x / 16, y / 16, delta_z]) * 16

                    if value < 0:
                        value = 0
                    else:
                        value = int(value)
                    # if value < 0:
                        # print(value)
                    self.buffer.led_level_set(x, y, value)

            self.buffer.render(self.grid)

            delta_z += 0.0125

            await asyncio.sleep(0.05)

async def main():
    loop = asyncio.get_running_loop()
    app = PerinApp()

    def serialosc_device_added(id, type, port):
        print('connecting to {} ({})'.format(id, type))
        asyncio.create_task(app.grid.connect('127.0.0.1', port))

    serialosc = monome.SerialOsc()
    serialosc.device_added_event.add_handler(serialosc_device_added)

    await serialosc.connect()
    await loop.create_future()

if __name__ == '__main__':
    asyncio.run(main())
